/******************************************************************************
 JMaxFunc.cc

								The Maximum JFunction Class

	This class returns the maximum of its arguments.

	BASE CLASS = JNaryFunction

	Copyright  1995 by John Lindal. All rights reserved.

 ******************************************************************************/

#include <JMaxFunc.h>
#include <jAssert.h>

/******************************************************************************
 Constructor

 ******************************************************************************/

JMaxFunc::JMaxFunc()
	:
	JNaryFunction(kJMaxFuncNameIndex, kJMaxFuncType)
{
}

/******************************************************************************
 Destructor

 ******************************************************************************/

JMaxFunc::~JMaxFunc()
{
}

/******************************************************************************
 Copy constructor

 ******************************************************************************/

JMaxFunc::JMaxFunc
	(
	const JMaxFunc& source
	)
	:
	JNaryFunction(source)
{
}

/******************************************************************************
 Copy

	This provides a way to copy an object without knowing its exact class.

 ******************************************************************************/

JFunction*
JMaxFunc::Copy()
	const
{
	JMaxFunc* newFunction = new JMaxFunc(*this);
	assert( newFunction != NULL );
	return newFunction;
}

/******************************************************************************
 Evaluate

 ******************************************************************************/

JBoolean
JMaxFunc::Evaluate
	(
	JFloat* result
	)
	const
{
	// start with the value of the first argument

	if (GetArgCount() == 0 || !(GetArg(1))->Evaluate(result))
		{
		return kFalse;
		}

	// find the maximum of all the arguments

	const JSize argCount = GetArgCount();
	for (JIndex i=2; i<=argCount; i++)
		{
		JFloat argValue;
		if (!(GetArg(i))->Evaluate(&argValue))
			{
			return kFalse;
			}
		if (argValue > *result)
			{
			*result = argValue;
			}
		}
	return kTrue;
}

JBoolean
JMaxFunc::Evaluate
	(
	JComplex* result
	)
	const
{
	// start with the value of the first argument

	if (GetArgCount() == 0 || !(GetArg(1))->Evaluate(result))
		{
		return kFalse;
		}

	// find the maximum real part of all the arguments

	const JSize argCount = GetArgCount();
	for (JIndex i=2; i<=argCount; i++)
		{
		JComplex argValue;
		if (!(GetArg(i))->Evaluate(&argValue))
			{
			return kFalse;
			}
		if (real(argValue) > real(*result))
			{
			*result = argValue;
			}
		}

	*result = real(*result);
	return kTrue;
}

/******************************************************************************
 SameAs

	Returns kTrue if the given function is identical to us.

 ******************************************************************************/

JBoolean
JMaxFunc::SameAs
	(
	const JFunction& theFunction
	)
	const
{
	return JNaryFunction::SameAsAnyOrder(theFunction);
}
